function love.load()
	local glsl = love.graphics._effectCodeToGLSL
	function love.graphics._effectCodeToGLSL(code)
		local i = 1
		for l in code:gmatch("[^\n]*\n") do
			io.write(("%03d:    %s"):format(i,l))
			i = i + 1
		end
		return glsl(code)
	end

	texture = love.graphics.newImage('Water_Texture_by_Jshei.jpg')
	dummy = love.graphics.newImage(love.image.newImageData(1,1))
	effect = love.graphics.newPixelEffect [[
extern number time = .0;
extern vec3 eye = vec3(.0,.0,-1.);
extern vec3 eps = vec3(.00001,.0,.0);
extern Image texture;

number s,c;

// scene definition
#define sphere(p, r) (length(p) - r)
#define h(p) \
	 min(sphere(p - vec3(1. + c*.5, .2 + s*.5, 5.), 1.7), \
	     sphere(p - vec3(-2.,.5, 6.+c), 1.5))

// sorry...
vec4 raycast(vec3 p, vec3 ray, number delta)
{
	number d;
#define STEP() d = h(p); if (d <= delta) return vec4(p,d); p+= ray * d
	STEP(); STEP(); STEP(); STEP(); STEP(); STEP();
	STEP(); STEP(); STEP(); STEP(); STEP(); STEP();
	STEP(); STEP(); STEP(); STEP(); STEP(); STEP();
	return vec4(p,d);
}

#define grad(f, p) normalize(vec3( f(p+eps) - f(p-eps), f(p+eps.yxz) - f(p-eps.yxz), f(p+eps.zyx) - f(p-eps.zyx) ));

vec4 effect(vec4 _unused_color_, Image _unused_image_, vec2 tc, vec2 _unused_pixel_coords_)
{
	s = sin(time); c = cos(time);

	tc = 2.0 * tc - vec2(1.0);
	vec3 ray = normalize(vec3(tc.x, tc.y, .0) - eye);

	vec4 info = raycast(eye, ray, .01);
	number d = info.w;
	if (d > .01) discard;

	vec3 p = info.xyz;
	vec3 n = grad(h, p);

	// fast fake ambient occlusion
	number delta = 1.;
	number ao = .1;
	ao += (1. * delta - h(p + n * 1. * delta)) / pow(2.0, 1.);
	ao += (2. * delta - h(p + n * 2. * delta)) / pow(2.0, 2.);
	ao += (3. * delta - h(p + n * 3. * delta)) / pow(2.0, 3.);
	ao = 1.0 - .3 * ao;

	// use normal for texture coordinates
	tc = n.zy * .5 + vec2(.5);
	return Texel(texture, tc) * vec4(ao,ao,ao, 1.0);
}

]]

	effect:send('texture', texture)
end

t = 0
ft,frames,fps = 0, 0, 0
function love.update(dt)
	-- more reliable fps counter
	frames = frames + 1
	ft = ft + dt
	if ft >= .25 then
		fps = math.floor(frames / ft)
		ft = ft - .25
		frames = 0
	end

	dt = math.min(dt, 1/30)
	t = t + dt
	effect:send('time', t)
end

function love.draw()
	love.graphics.setPixelEffect(effect)
	love.graphics.draw(dummy, 0,0, 0, love.graphics.getWidth(), love.graphics.getHeight())
	love.graphics.setPixelEffect()
	love.graphics.print("FPS: " .. fps, 2,2)
end
